import os, pickle
import numpy as np
from rdkit import Chem
from keras.models import model_from_json
from pkg_resources import resource_filename, Requirement
import openbabel


def loadKerasModel(path,name):    
    """
    function to load a pre-trainned
    keras neural network model 
    
    :param: path: path to ANN folder (in package)
    :param: path: string, name of model
    :return: keras model (the neural network object)
    """ 
    # load model
    json_file = open(path+'/'+name+'_model.json', 'r')
    loaded_model_json = json_file.read()
    json_file.close()
    loaded_model = model_from_json(loaded_model_json)
    #load weights into new model
    loaded_model.load_weights(path+'/'+name+'_model.h5', 'r')
    loaded_model.compile(loss="mse",optimizer='sgd',
              metrics=['mse', 'mae', 'mape'])   
    return(loaded_model)


def loadNormalizationData(path,name):
    """
    function to load the trainning
    data-scaling numbers needed to keep
    consistent with training
        
    :param: path: path to ANN folder (in package)
    :param: path: string, name of model
    :return: np.arrays for trainning mean/var data
    """ 
    train_mean_x = list()
    path_to_file = path + name + '_mean_x.csv'
    if os.path.isfile(path_to_file):
        with open(path_to_file, 'r') as f:
            for lines in f.readlines():
                train_mean_x.append([float(lines.strip().strip('[]'))])
    train_var_x = list()
    path_to_file = path + name + '_var_x.csv'
    with open(path_to_file, 'r') as f:
        for lines in f.readlines():
            train_var_x.append([float(lines.strip().strip('[]'))])
    train_mean_y = list()
    path_to_file = path + name + '_mean_y.csv'
    with open(path_to_file, 'r') as f:
        for lines in f.readlines():
            train_mean_y.append([float(lines.strip().strip('[]'))])
    train_var_y = list()
    path_to_file =path + name + '_var_y.csv'
    with open(path_to_file, 'r') as f:
        for lines in f.readlines():
            train_var_y.append([float(lines.strip().strip('[]'))])
    
    train_mean_x = np.array(train_mean_x)
    train_var_x = np.array(train_var_x)
    train_mean_y = np.array(train_mean_y)
    train_var_y = np.array(train_var_y)

    return train_mean_x, train_mean_y, train_var_x, train_var_y

def loadANNModel():
    """
    function to load the saved
    neural network model and parameters
    this load can be the slowest
    part of the whole operation so
    should only be once off
    
    :return: dictionary, containing loaded data
    """ 
    
    # get path info from package manager
    ofrpath = resource_filename(Requirement.parse("ofrcalc"),"ofrcalc/data/ofrmod")
    
    # load the ANN model and normalzation data
    
    model = loadKerasModel(path=ofrpath, name='single-ANN')
    train_mean_x, train_mean_y, train_var_x, train_var_y = loadNormalizationData(path=ofrpath+'/', name='single-ANN')
    feature_names = pickle.load(open(ofrpath+"/feature_names.p", "rb"))
    NNdict = {"model":model,"feature_names":feature_names,
              "train_mean_x":train_mean_x,"train_mean_y":train_mean_y,
              "train_var_x":train_var_x,"train_var_y":train_var_y}
    return(NNdict)

def loadInputFile(inpath):
    """
    function to read the user
    supplied input file and check
    that it can be understood by the 
    code
    
    :param: inpath: path to input file
    :return: dictionary, containing loaded data
    """ 
    inputs = []
    errors = []
    valid = True
    with open(inpath,'r') as f:
        if valid:
            for i, lines in enumerate(f.readlines()):
                ll = lines.strip().split(' ')
                if not len(ll) == 4:
                    errors.append('error : line number '+str(i) + ' is the wrong length')
                    valid = False
                    thisInput = {}
                else:
                    name, smiles, con1, con2 = ll
                    try: 
                        con1 = int(con1)
                        con2 = int(con2)
                    except:
                        errors.append('error : line number '+str(i) + ' connection atom was not understood')
                        valid = False

                    thisInput = {"name":name,'smiles':smiles,'conIdx': [con1, con2 ]}
                    if valid:
                        valid, valErrors = inputValidator(thisInput)
                if not valid:
                    print(errors)
                    print(valErrors)
                    sys.exit()
                else:
                    inputs.append(thisInput)
                    msg='valid input detected at line number '+str(i) 
                    print(msg)
    return(inputs)

def gradientPicSave(results, picDir):
    """
    function to take results, including gradient info
    from procsmiles.invokeFromSmilles and save the images
    into the indicated directory
    :param: results: dictionary, prepared by procsmiles.invokeFromSmiles
    :param: picDir: path to dir
    """
    
    if not os.path.isdir(picDir):
        os.makedirs(picDir)
    for pic,l in zip(results['gradPics'],results['baseNames']):
        with open(picDir+l+'.png','wb') as f:
            f.write(pic)
    
def resultWriter(resultsList, resDir):
    """
    function to take results
    from procsmiles.invokeFromSmilles and save
    COF inro into the indicated directory
    :param: results: dictionary, prepared by procsmiles.invokeFromSmiles
    :param: resDir: path to dir
    """ 
    if not os.path.isdir(resDir):
        os.mkdir(resDir)   
    ## make results table
    with open(resDir.strip('/') + '/results.csv','w') as wf:
        wf.write(','.join(resultsList[0]["conditionNames"])+',COF\n') 
        for rn in resultsList:
                 for i, cn in enumerate(rn["conditions"]):
                     wf.write(','.join([str(j) for j in cn])+  ',' + str(np.round(rn["predictions"][i][0],4))+'\n')
def resultRead(path):
    """
    function to read results from resultWriter
    and return the average COF and # of results. This
    is intended for use in testing only. 
    :param: path: path to results file
    :return: int, float number of and mean COF in file
    """ 
    # list to hold COFs
    COFs = []
    with open(path, 'r') as f:
        for i, l in enumerate(f.readlines()):
            if i > 0:
                ll = l.strip().split(",")
                COFs.append(float(ll[-1]))
    return(len(COFs),np.mean(COFs))
    
    
def inputValidator(ins):
    """
    function to test and validate
    inputs harvested from infiles and provide 
    useful errors to the user
    
    :param: ins: dictionary, containing info from one line of input
    :return: bool, list of errors the status of the input
    """ 
    errors = []
    valid = True
    
    # test presence of keys:
    for key in ['smiles','name','conIdx']:
        if not key in ins.keys():
            errors.append('error: key '+ key +' missing from input')
            valid = False
    if not valid:
        return(valid, errors)
    else:
        # check smiles
        try:
            rmdol = Chem.MolFromSmiles(ins['smiles'])
        except:
            errors.append('error: SMILEs '+ str(ins['smiles']) +' not able to be understood by rdkit')
            valid = False
        # check conIdx
        conIdx = ins['conIdx']
        if not len(conIdx) == 2:
            errors.append('error: OFRCalc is trained on bidentate binding.'\
                          'Instead '+ str(len(ins['conIdx'])) +' atoms were given. Please choose 2 instead.')
            valid = False
    return(valid, errors)


def velocityValidator(args):
    """
    function to test and validate
    velocity inputs provied at command line
    and provide
    useful errors to the user
    
    :param: args, the argparse object
    :return: np.array, velocity range for ANN eval
    :return: bool, is the input valid
    :return: list of errors the status of the input
    """ 
    vrange = []
    errors = []
    valid = True
    
    # test presence of keys:
    for arg in ['vlow','vhigh','vsteps']:
        if not hasattr(args, arg):
            errors.append('error: arg '+ arg +' missing from input')
            valid = False
    if not valid:
        return(vrange,valid, errors)
    else:
        # check the range
        if args.vhigh - args.vlow < 0:
            errors.append('error: vhigh is smaller than vsmall. Please correct')
            valid = False
        # check they are in within sane limits
        if args.vhigh > 10.5 or args.vhigh <0.5:
            errors.append('error: vhigh out of range! must be <= 10.5 m/s and > 0.5 m/s, training data only extends to 10 m/s.')
            valid = False
        if args.vlow < 0.25 or args.vlow >10.0:
            errors.append('error: vlow out of range! must be >= 0.25 m/s and < 10.0 m/s, training data only extends to 0.5 m/s.')
            valid = False
        # check vsteps
        if args.vsteps > 1000 or args.vsteps  < 1:
            errors.append('error: vsteps out of range! must be >= 1 and <= 1000.')
            valid = False
    if valid:
        vrange = np.linspace(start=args.vlow,stop=args.vhigh,num=args.vsteps)
        
    return(vrange, valid, errors)
   
#count bonds of an atom
def bondCount(aa):
    bCount = 0
    for b in openbabel.OBAtomBondIter(aa):
        bCount = bCount + 1
    return bCount

#find acceptable pairs
def find_pairs(conlist):
    """
    function finds out the possible pairs for
    connection atoms
    if conlist length < 2, monodentate
    if conlist length >= 2 and pair distance > 4, monodentate
    if conlist length >= 2 and pair distance <= 3, list all pairs
    """
    lconlist = len(conlist)
    pairlist = []
    if lconlist < 2:
        pairlist.append([conlist[0],conlist[0]])
    else:
        for i in range(0,lconlist-1):
            if abs(conlist[i]-conlist[i+1]) > 4:
                pairlist.append([conlist[i],conlist[i]])
            if abs(conlist[i]-conlist[i+1]) <= 4:
                pairlist.append([conlist[i],conlist[i+1]])

    return pairlist

#find non-C atoms
def find_nonCAtoms(obmol):
    """
    function takes the SMILES
    return: A list of nonC atoms indeses
    """
    nonC = []
    for obatom in openbabel.OBMolAtomIter(obmol):    
        if obatom.GetAtomicNum() != 6:
            nonC.append([obatom.GetAtomicNum(),obatom.GetIdx(),bondCount(obatom)])
    # remove non possible cases        
    nonCindx = []
    for item in nonC:
        if item[0] == 8 and item[2] > 1:
            pass
        if item[0] == 7 and item[2] > 1:
            pass
        else:
            nonCindx.append(item[1]-1)
    return nonCindx

def filt_out(mystr):
    newstr = ""
    for i in range(0,len(mystr)):
        if mystr[i] not in ['=','-','(',')','[',']','1','2','.','/']:
            newstr = newstr+mystr[i]
    return newstr

def genInputFile(filename):
    hgSMARTS=['[#6](-[#8])=[#8]',
       '[#6](-[#8])-[#8]',
        '[#6](-[#8])-[#7]', # alc-amide
        '[*](-[#8])-[#8]', 
        '[#6](-[#7])-[#7]', # amide-amide
       '[#6](-[#7])=[#8]', # carbonyl-amide
         '[#6](-[#6]-[#8])-[#8]',
         '[#6](-[#6]-[#8])=[#8]',
         '[#6](-[#6]=[#8])=[#8]',
         '[#6](-[#6]-[#7])-[#8]',
         '[#6](-[#6]-[#7])=[#8]',
         '[#6](-[#6]-[#7])-[#7]',
         '[#6](-[#6]-[#8])-[#6]-[#8]',
         '[#6](-[#6]-[#8])-[#6]=[#8]',
         '[#6](-[#6]=[#8])-[#6]=[#8]']

    cdxml = filename
    obconv = openbabel.OBConversion()  # ob Class
    obmol = openbabel.OBMol()  # ob Class
    obconv.SetInAndOutFormats('cdxml','smi')  # ob Method to set cdxml
    obconv.ReadFile(obmol, cdxml)  # ob Method to read cdxml into a OBMol()
    obSMI = obconv.WriteFile(obmol,filename+'.smi')
    obconv.SetInFormat('smi')
    obconv.ReadFile(obmol, filename+'.smi')

    with open(filename+'.smi','r') as f:
        for line in f.readlines():
            a=line.split()
    baseSMI = a[0]

    m1 = Chem.MolFromSmiles(baseSMI)
    
    condict = {}
    counter = 0
    for patt in hgSMARTS:
        ipatt = Chem.MolFromSmarts(patt)
        if m1.HasSubstructMatch(ipatt):
            conlist = []
            for i in m1.GetSubstructMatch(ipatt):
                atom  = m1.GetAtomWithIdx(i)
                if atom.GetSymbol() in ['O','N'] and len(atom.GetBonds()) == 1.0:
                    conlist.append(i)
            if conlist:        
                condict.update({counter:conlist})
            counter = counter + 1
    with open(filename.split('.')[0]+'.txt','w') as wf:
        for i in condict:
            pair = condict[i]
            if len(pair) == 1:
                wf.write('mol'+str(i)+' '+baseSMI+' '+str(pair[0])+' '+str(pair[0])+'\n')
            elif len(pair) == 2:
                wf.write('mol'+str(i)+' '+baseSMI+' '+str(pair[0])+' '+str(pair[1])+'\n')
            else:
                smi=filt_out(baseSMI)
                id1 = len(smi)-2
                id2 = len(smi)-1
                wf.write('mol0 '+baseSMI+' '+str(id1)+' '+str(id2)+'\n')




## @file atom3D.py
#  Defines atom3D class and contains useful manipulation/retrieval routines.
#
#  Written by Tim Ioannidis and JP Janet for HJK Group
#
#  Dpt of Chemical Engineering, MIT

from math import sqrt 
from ofrcalc.classes.globalvars import globalvars

## Class for atoms that will be used to manipulate coordinates and other properties
class atom3D:
    ## Constructor
    #  @param self The object pointer    
    #  @param Sym Element symbol
    #  @param xyz List of coordinates
    #  @param name optional unique identifier
    def __init__(self,Sym='C',xyz=[0.0,0.0,0.0],name=False): 
		## Element symbol
        self.sym = Sym
        globs = globalvars()
        amass = globs.amass()
        if Sym not in amass: # assign default values if not in dictionary
            print(("We didn't find the atomic mass of %s in the dictionary. Assigning default value of 12!\n" %(Sym)))
            ## Atomic mass
            self.mass = 12 # default atomic mass
            ## Atomic number
            self.atno = 6 # default atomic number
            ## Covalent radius
            self.rad = 0.75 # default atomic radius
        else:
            self.mass = amass[Sym][0]
            self.atno = amass[Sym][1]
            self.rad = amass[Sym][2]
        ## Flag for freezing in optimization
        self.frozen =  False
        ## Flag for atom name 
        if name:
            self.name = name
        else:
            self.name = Sym
        ## flag for metal
        self.metal = None
            
        ## Coordinates
        self.__xyz = xyz
        
    ## Get coordinates
    #  @param self The object pointer
    #  @return List of coordinates
    def coords(self): 
        x,y,z = self.__xyz
        return [x,y,z]
        
    ## Get distance with another atom
    #  @param self The object pointer
    #  @param atom2 atom3D of second atom
    #  @return Distance scalar
    def distance(self,atom2):
        xyz = self.coords()
        point = atom2.coords()
        dx = xyz[0]-point[0]
        dy = xyz[1]-point[1]
        dz = xyz[2]-point[2]
        return sqrt(dx*dx+dy*dy+dz*dz)
        
    ## Get distance vector with another atom
    #  @param self The object pointer
    #  @param atom2 atom3D of second atom
    #  @return Distance vector
    def distancev(self,atom2):
        xyz = self.coords()
        point = atom2.coords()
        dx = xyz[0]-point[0]
        dy = xyz[1]-point[1]
        dz = xyz[2]-point[2]
        return [dx,dy,dz]
        
    ## Check if atom is a metal
    #  @param self The object pointer
    #  @return ismetal bool
    def ismetal(self):
        if self.metal is None:
            if self.sym in globalvars().metals():
                self.metal = True
            else:
                self.metal = False
        return self.metal
            
    ## Set 3D coordinates
    #  @param self The object pointer
    #  @param xyz Coordinate vector
    def setcoords(self,xyz):
        self.__xyz[0] = xyz[0]
        self.__xyz[1] = xyz[1]
        self.__xyz[2] = xyz[2]
        
    ## Get symbol
    #  @param self The object pointer
    #  @return Element symbol
    def symbol(self):
        return self.sym

    ## mutate
    #  @param self The object pointer
    #  @parm newType the new atom type
    def mutate(self,newType):
        globs = globalvars()
        amass = globs.amass()
        if newType not in list(amass.keys()):
           print(('Error, unknown atom atom type transformation to ' + str(newType)))
           print('no changes made')
        else:
            self.mass = amass[newType][0]
            self.atno = amass[newType][1]
            self.rad = amass[newType][2]
            self.name = newType
            self.sym = newType

    ## Translate atom
    #  @param self The object pointer
    #  @param dxyz Distance vector to translate
    def translate(self,dxyz):
        x,y,z = self.__xyz
        self.__xyz[0] = x + dxyz[0]
        self.__xyz[1] = y + dxyz[1]
        self.__xyz[2] = z + dxyz[2]
        
    ## Print methods
    #  @param self The object pointer
    #  @return String with methods
    def __repr__(self):
        ss = "\nClass atom3D has the following methods:\n"
        for method in dir(self):
            if callable(getattr(self, method)):
                ss += method +'\n'
        return ss
    




from ofrcalc.routines.autocorrelation import*
import time
from rdkit import Chem


class quickOFR:
    def __init__(self,name,mol,conIdx):
        """
        constructor for OFR class
        :param: name str name of ofr
        :param: mol  mol3D object for ofr
        :param: conIdx  list of int, connect atom indices
        """
        self.name =  name
        self.mol = mol
        self.conIdx = conIdx
        self.smiles = False
        ## descriptors
        self.set_desc = False
        self.descriptors =  list()
        self.descriptorNames =  list()
        self.descriptorDerivatives =  None
        self.descriptorDerivativeNames =  list()
        self.rdmol =  False
        
        
    def setRdMolFromSmiles(self, smiles):
        """
        creates RDKit mol object from smiles
        :param: smiles str SMILES string for OFR
        """
        # save the SMILEs to the rdmol objet
        self.smiles = smiles
        # save this rdmol for drawing later
        self.rdmol =  Chem.MolFromSmiles(self.smiles) 
    
    def getDescriptorVector(self,loud=False,name=False):
        """
        computes molecular AC fingergrpint derivatives
        :param: loud bool print debug info
        """
        ## full ACs
        resultsDictionary = generate_full_complex_autocorrelations(self.mol,depth=4,loud=loud,oct=False)
        self.appendDescriptors(resultsDictionary['colnames'],resultsDictionary['results'],'f','all')
        ## connennection atom acs 
        resultsDictionary = generate_atomonly_autocorrelations(self.mol,self.conIdx,depth=5,loud=loud,oct=False)
        self.appendDescriptors(resultsDictionary['colnames'],resultsDictionary['results'],'con','all')
        
    def appendDescriptors(self,list_of_names,list_of_props,prefix,suffix):
        """
        adds the AC molecular fingerprints to the object
        :param: list_of_names list of names of features
        :param: list_of_props list of descriptors
        :param: prefix str prefix for AC names
        :param: suffix str suffix for AC names
        """  
        for names in list_of_names:
            if hasattr(names, '__iter__'):
                names = [".".join([prefix,str(i).replace('-','.'),suffix]) for i in names]
                self.descriptorNames += names
            else:
                names = ".".join([prefix,str(names).replace('-','.'),suffix])
                self.descriptorNames.append(names)
        for values in list_of_props:
            if hasattr(values, '__iter__'):
                self.descriptors.extend(values)
            else:
                self.descriptors.append(values)

    def getDescriptorVectorDerivatives(self,loud=False):
        """
        computes molecular AC fingergrpint derivatives
        :param: loud bool print debug info
        """
        ## full ACs
        resultsDictionary = generate_full_complex_autocorrelation_derivatives(self.mol,depth=4,loud=loud,oct=False)
        self.appendDescriptorDerivatives(resultsDictionary['colnames'],resultsDictionary['results'],'f','all')
        ## connennection atom acs 
        resultsDictionary = generate_atomonly_autocorrelation_derivatives(self.mol,self.conIdx,depth=5,loud=loud,oct=False)
        self.appendDescriptorDerivatives(resultsDictionary['colnames'],resultsDictionary['results'],'con','all')
    
    def appendDescriptorDerivatives(self,mat_of_names,dmat,prefix,suffix):
        """
        adds the AC molecular fingerprint gradients to the object
        :param: mat_of_names list of list of names of features
        :param: dmat np array of descriptors
        :param: prefix str prefix for AC names
        :param: suffix str suffix for AC names
        """  
        for names in mat_of_names:
            jnames = [".".join([prefix,str(i).replace('-','.'),suffix]) for i in names]
            self.descriptorDerivativeNames.append(jnames)
        if self.descriptorDerivatives is None:
            self.descriptorDerivatives = dmat
        else:
            self.descriptorDerivatives =  np.row_stack([self.descriptorDerivatives,dmat]) 
    
    def getBondCounts(self):
        """
        Find the count of  C=C double bonds and 
        CO groups in the SMILES string using rdkit
        """  
        dbBondProbe = Chem.MolFromSmarts('C=C')
        coBondProbe = Chem.MolFromSmarts('C=O')       
        
        # use rd subtructure to find out if there are double bonds/CO groups
        dbCount  = len(self.rdmol.GetSubstructMatches(dbBondProbe))
        coCount = len(self.rdmol.GetSubstructMatches(coBondProbe))
        # add to vector
        self.descriptors.extend([dbCount,coCount])
        self.descriptorNames.extend(['db_count','co_count'])
        
    def filt_out(self,mystr):
        """
        Filter out the special charectors for recognizing specific
        patterns
        """
        newstr = ""
        for i in range(0,len(mystr)):
            if mystr[i] not in ['=','-','(',')','[',']','1','2','.','/']:
                newstr = newstr+mystr[i]
        return newstr
        
    def getSidechainCounts(self):
        """
        Find the count of side chains in the SMILES string
        using rdkit
        """
                       
        n_chain = 0
        list_sidechain = []
        newstr = ""
        j = 0

        if self.rdmol.GetRingInfo().NumRings() > 0:
            n_chain = 1
        else:
            for i in range(0,len(self.smiles)):
                if self.smiles[i] == '(':
                    j = i + 1
                    while self.smiles[j] != ')':
                        newstr = newstr+self.smiles[j]
                        j = j + 1
                    list_sidechain.append(newstr)
                    i = j
            for i,sub in enumerate(tuple(list_sidechain)):

                newsub = self.filt_out(sub)
                if 'O' in sub:
                        print('removing')
                        list_sidechain.remove(sub)
            n_chain = len(list_sidechain)
        
        self.descriptors.extend([n_chain])
        self.descriptorNames.extend(['Number.of.tail.side.chains'])



# function to handle parsing CLI arguments and checking on them 

import argparse


def createParser():
     """
     function to create argument parser for easy CLI access
     to the module and automatic creation of help text
     :return: argprase parser object
     """
     parser = argparse.ArgumentParser(
     description='Predict NEMD coefficient based on SMILEs strings.')

     parser.add_argument(dest='infile',  type=str, 
                    help='input file. see docs for syntax')
     parser.add_argument('--vlow',  type=float, 
                    help='minimun value for sliding velocity',
                    default = 0.5,nargs='?')                    
     parser.add_argument('--vhigh',  type=float, 
                    help='max value for sliding velocity',
                    default = 10.0,nargs='?')
     parser.add_argument('--vsteps',  type=int, 
                    help='steps to take between vlow and vhigh',
                    default = 5,nargs='?')                    
     parser.add_argument('--res', dest='resDir', type=str,
                         default='res/',
                         help='path to folder to save results')
     parser.add_argument('--pic', dest='picDir', type=str,
               default='gradientpics/', help='path to folder to save gradient \
               pictures, if draw is on')
     parser.add_argument('--draw', dest='getGradients',
                help='draw gradient pictures',  action="store_true")
     parser.add_argument('--signed', dest='absGradients',
                help='used signed  gradients',  action="store_false")
     return(parser)



import matplotlib.pyplot as plt
import matplotlib.cm as cm

import tensorflow as tf
import openbabel as ob

import tkinter as tk
from tkinter import filedialog

import os, glob
from PIL import Image, ImageTk

import numpy as np

from ofrcalc.routines.iomanager import loadInputFile, velocityValidator, genInputFile  
from ofrcalc.routines.iomanager import resultWriter, gradientPicSave, loadANNModel
from ofrcalc.routines.procsmiles import invokeFromSmiles



def main():


    def generate_COF_graph(resultsList):

        """
        Draws a simple lineplot of the COF vs velocity
        for each OFR-temperature combinations        
        """
        # draw a graph of velocity vs COF
        
        # organize the data
        plotting_results = {}
        for results in resultsList:
            velos =  [float(cn[3]) for  i,cn in enumerate(results["conditions"]) if cn[5] == "medium"]
            temps =  [float(cn[4]) for  i,cn in enumerate(results["conditions"]) if cn[5] == "medium"]
            COFs =  [results["predictions"][i][0] for i,cn in enumerate(results["conditions"]) if cn[5] == "medium"]
            for tt in set(temps):
                unique_name = results["conditions"][0][0] + '@' + str(int(tt)) + 'C'
                this_v = [velos[i] for i,t in enumerate(temps) if t == tt]
                this_cof = [COFs[i] for i,t in enumerate(temps) if t == tt]
                plotting_results.update({unique_name:{'velos':this_v,'COFs':this_cof}})
                
        # qualitative colors to max readability                    
        number_of_colors = len(plotting_results.keys())
        smap = plt.get_cmap("Set2",number_of_colors)

        # draw the plot
        for i, keys in enumerate(plotting_results.keys()):
            plt.plot(plotting_results[keys]['velos'],plotting_results[keys]['COFs'],
                     label=keys,color=smap(i))
        plt.legend()
        plt.savefig('testfig.png',res=300,width=3.33,height=2.9,unit='in')
        plt.close()

    def open_file():
        
        filename = filedialog.askopenfilename(initialdir="./",title = "Select file",
                                                     filetypes = (("text files","*.txt"),("cdxml files","*.cdxml"),("all files","*.*")))
        # ("cdxml files","*.cdxml")
        return filename

    def openfile_display():
        
        fname = open_file()
        
        # This creates the text field
        #greeting_display = tk.Text(master=window, height=10,width=30)
        entry1.delete(0, "end") 
        entry1.insert(tk.END,fname)
        
    def read_file():
        
        filename=str(entry1.get())
        #with open(filename,'r') as mydata:
        #   lines=mydata.read()
        #load_cdxml(filename)
        if filename.split('.')[1] == 'cdxml':
            genInputFile(filename)
            filename=filename.split('.')[0]+'.txt'
        with open(filename,'r') as output:
            lines=output.read()
        
        filecontent_display = tk.Text(master=window, height=10,width=30)
        filecontent_display.grid(column=0, row=3)
        
        filecontent_display.insert(tk.END,lines)
        filecontent_display.config(state='disable')
        
    def execute_ofrcalc():
        
        filename=str(entry1.get())
        if filename.split('.')[1] == 'cdxml':
            genInputFile(filename)
            filename=filename.split('.')[0]+'.txt'
        foldername=filename.split('.')[0]
        foldername=foldername.split('/')[len(foldername.split('/'))-1]
       
        ## start with loading the model
        NNdict = loadANNModel()
        print('loaded model successfully')
        
        
        ## load the input file
        inputList  = loadInputFile(filename)
        
        ## holder for results 
        resultsList = []
        
        ## loop over inputs
        thisPicdir = foldername+'-gui-pics/'
        for i, ins in enumerate(inputList):
            ins.update({'getGradients':True})
            ins.update({'absGradients':False})
            ins.update({'NNdict':NNdict})
            ins.update({'vrange':np.linspace(0.5,10,5)})
            results = invokeFromSmiles(**ins)
            resultsList.append(results)
            gradientPicSave(results, picDir = thisPicdir)
        
        generate_COF_graph(resultsList)
        
        thisResdir = foldername+'-gui-res/'
        resultWriter(resultsList,thisResdir)
        
        
        
        
        #myCmd = 'ofrcalc ' + filename
        #print(myCmd)
        #subprocess.run(myCmd,shell=True)
        
        
        res_name = foldername+'-gui-res/results.csv'
        with open(res_name,'r') as res_data:
            lines = res_data.read()
        
        filecontent_display = tk.Text(master=window, height=10,width=100)
        filecontent_display.grid(column=2, row=3)
        S = tk.Scrollbar(window, orient="vertical", command=filecontent_display.yview)
        S.grid(column=3,row=3,sticky='ns')
        filecontent_display.config(yscrollcommand=S.set)
        
        #horizontal sliding bar not working
        #S1 = tk.Scrollbar(window, orient="horizontal", command=filecontent_display.xview)
        #S1.grid(column=2,row=4,sticky='ew')
        #filecontent_display.config(xscrollcommand=S1.set)
        
        filecontent_display.insert(tk.END,lines)
        filecontent_display.config(state='disable')
        

        load2 = Image.open("testfig.png")
        width,height=load2.size
        print([width,height])
        #load2 = load2.resize((100,30))
        render2 = ImageTk.PhotoImage(load2)
        struct_pic_display2 = tk.Label(image=render2)
        struct_pic_display2.image = render2
        struct_pic_display2.grid(column=2,row=4)

        picglob = glob.glob(thisPicdir+'*.png')   
        load3 = Image.open(picglob[0])
        #load3 = load3.resize((100,30))
        width,height=load3.size
        print([width,height])
        render3 = ImageTk.PhotoImage(load3)
        struct_pic_display3 = tk.Label(image=render3)
        struct_pic_display3.image = render3
        struct_pic_display3.grid(column=0,row=4)
        
        
        #display a molecule picture
    #   load1 = Image.open("examplePic.png")
    #   width,height=load1.size
    #   #load = load.resize((40,20))
    #   render1 = ImageTk.PhotoImage(load1.resize((round(width/2),round(height/2))))
    #   struct_pic_display1 = tk.Label(image=render1)
    #   struct_pic_display1.image = render1
 
        
    def load_cdxml(cdxml):
        obconv = openbabel.OBConversion()  # ob Class
        obmol = openbabel.OBMol()  # ob Class
        obconv.SetInFormat('cdxml')  # ob Method to set cdxml
        obbconv.SetOutFormat('SMILES')
        obconv.ReadFile(obmol, cdxml)  # ob Method to reaad cdxml into a OBMol()
        
        mol = mol3D()
        mol.OBMol = obmol
        mol.convert2mol3D()
    

    # squelch some TF output
    os.environ['TF_CPP_MIN_LOG_LEVEL'] = '3'
    os.environ['KMP_WARNINGS'] = '0'
    os.environ['TF_CPP_MIN_LOG_LEVEL'] = '3'
    tf.compat.v1.logging.set_verbosity(tf.compat.v1.logging.ERROR)
    try:
        from tensorflow.python.util import module_wrapper as deprecation
    except ImportError:
        from tensorflow.python.util import deprecation_wrapper as deprecation
    deprecation._PER_MODULE_WARNING_LIMIT = 0
   
    # squelch some OB output
    msghdlr = ob.OBMessageHandler()
    msghdlr.SetOutputLevel(0)
    
    
    window = tk.Tk()
    window.title("OFR Calculator")
    window.geometry("1000x600")    
    #------LABEL------
    label1 = tk.Label(text = "Welcome to OFR friction coefficients calculator v1.0!")
    label1.grid(column=0,row=0)

    label2 = tk.Label(text = "Import SMILES files or ChemDraw files ")
    label2.grid(column=0,row=1)

    #------ENTRIES------
    entry1 = tk.Entry()
    entry1.grid(column=1,row=1)

    #------BUTTON------
    botton1 = tk.Button(text = "Browse...", command=openfile_display)
    botton1.grid(column=2,row=1)

    botton2 = tk.Button(text = "Import", command=read_file)
    botton2.grid(column=1,row=2)

    botton3 = tk.Button(text = "Calculate", command=execute_ofrcalc)
    botton3.grid(column=1,row=3)

    #------

    window.mainloop()

if __name__ == '__main__':
    main()
#main()




import sys
import os
import numpy as np
import math
import random 
from ofrcalc.classes.atom3D import *
from ofrcalc.classes.mol3D import *
from ofrcalc.classes.globalvars import globalvars



########### UNIT CONVERSION
HF_to_Kcal_mol = 627.503


def autocorrelation(mol, prop_vec, orig, d, oct=True, catoms=None):
    ## this function returns the autocorrelation
    ## for one atom
    # Inputs:
    #	mol - mol3D class
    #	prop_vec - vector, property of atoms in mol in order of index
    #	orig -  int, zero-indexed starting atom
    #	d - int, number of hops to travel
    #	oct - bool, if complex is octahedral, will use better bond checks
    result_vector = np.zeros(d + 1)
    hopped = 0
    active_set = set([orig])
    historical_set = set()
    result_vector[hopped] = prop_vec[orig] * prop_vec[orig]
    while hopped < (d):

        hopped += 1
        new_active_set = set()
        for this_atom in active_set:
            ## prepare all atoms attached to this connection
            # print('called in AC')
            this_atoms_neighbors = mol.getBondedAtomsSmart(this_atom, oct=oct)
            for bound_atoms in this_atoms_neighbors:
                if (bound_atoms not in historical_set) and (bound_atoms not in active_set):
                    new_active_set.add(bound_atoms)
        # print('new active set at hop = ' +str(hopped) + ' is ' +str(new_active_set))
        for inds in new_active_set:
            result_vector[hopped] += prop_vec[orig] * prop_vec[inds]
            historical_set.update(active_set)
        active_set = new_active_set
    return (result_vector)

def autocorrelation_derivative(mol, prop_vec, orig, d, oct=True, catoms=None):
    ## this function returns the derivative vector
    ## of the scalar autocorrelation 
    ## starting at orig with depth d,
    ## with respect to the atomic properties
    ## in prop_vec, for all atoms.
    ## The return type is np.array
    ## Be sure to read this carefully!
    ## for one atom
    # Inputs:
    #	mol - mol3D class
    #	prop_vec - vector, property of atoms in mol in order of index
    #	orig -  int, zero-indexed starting atom
    #	d - int, number of hops to travel
    #	oct - bool, if complex is octahedral, will use better bond checks
    derivative_mat = np.zeros((d + 1,len(prop_vec)))
    
    # loop for each atom

    hopped = 0
    active_set = set([orig])
    historical_set = set()
    for derivate_ind in range(0,len(prop_vec)):
        if derivate_ind == orig:
            derivative_mat[hopped,derivate_ind] = 2*prop_vec[orig]
        else:
            derivative_mat[hopped,derivate_ind] = 0
    while hopped < (d):

        hopped += 1
        new_active_set = set()
        for this_atom in active_set:
            ## prepare all atoms attached to this connection
            # print('called in AC')
            this_atoms_neighbors = mol.getBondedAtomsSmart(this_atom, oct=oct)
            for bound_atoms in this_atoms_neighbors:
                if (bound_atoms not in historical_set) and (bound_atoms not in active_set):
                    new_active_set.add(bound_atoms)
        # print('new active set at hop = ' +str(hopped) + ' is ' +str(new_active_set))
        for inds in new_active_set:
            for derivate_ind in range(0,len(prop_vec)):
                if derivate_ind == orig:
                    derivative_mat[hopped,derivate_ind] += prop_vec[inds]
                elif derivate_ind == inds:
                    derivative_mat[hopped,derivate_ind] += prop_vec[orig]
            historical_set.update(active_set)
        active_set = new_active_set
    return (derivative_mat)
	





def full_autocorrelation(mol, prop, d, oct=oct, modifier= False):
    w = construct_property_vector(mol, prop, oct=oct, modifier=modifier)
    index_set = range(0, mol.natoms)
    autocorrelation_vector = np.zeros(d + 1)
    for centers in index_set:
        autocorrelation_vector += autocorrelation(mol, w, centers, d, oct=oct)
    return (autocorrelation_vector)

def full_autocorrelation_derivative(mol, prop, d, oct=oct, modifier= False):
    w = construct_property_vector(mol, prop, oct=oct, modifier=modifier)
    index_set = range(0, mol.natoms)
    autocorrelation_derivative_mat = np.zeros((d + 1,mol.natoms))
    for centers in index_set:
        autocorrelation_derivative_mat += autocorrelation_derivative(mol, w, centers, d, oct=oct)
    return (autocorrelation_derivative_mat)

def atom_only_autocorrelation(mol, prop, d, atomIdx, oct=True):
    ## atomIdx must b either a list of indcies
    ## or a single index
    w = construct_property_vector(mol, prop, oct)
    autocorrelation_vector = np.zeros(d + 1)
    if hasattr(atomIdx, "__len__"):
        for elements in atomIdx:
            autocorrelation_vector += autocorrelation(mol, w, elements, d, oct=oct)
        autocorrelation_vector = np.divide(autocorrelation_vector, len(atomIdx))
    else:
        autocorrelation_vector += autocorrelation(mol, w, atomIdx, d, oct=oct)
    return (autocorrelation_vector)

def atom_only_autocorrelation_derivative(mol, prop, d, atomIdx, oct=True):
    ## atomIdx must b either a list of indcies
    ## or a single index
    w = construct_property_vector(mol, prop, oct)
    autocorrelation_derivative_mat = np.zeros((d + 1,mol.natoms))
    if hasattr(atomIdx, "__len__"):
        for elements in atomIdx:
            autocorrelation_derivative_mat += autocorrelation_derivative(mol, w, elements, d, oct=oct)
        autocorrelation_derivative_mat = np.divide(autocorrelation_derivative_mat, len(atomIdx))
    else:
        autocorrelation_derivative_mat += autocorrelation_derivative(mol, w, atomIdx, d, oct=oct)
    return (autocorrelation_derivative_mat)
    

def atom_only_deltametric(mol, prop, d, atomIdx, oct=True,modifier=False):
    ## atomIdx must b either a list of indcies
    ## or a single index
    w = construct_property_vector(mol, prop, oct=oct,modifier=modifier)
    
    deltametric_vector = np.zeros(d + 1)
    if hasattr(atomIdx, "__len__"):
        for elements in atomIdx:
            deltametric_vector += deltametric(mol, w, elements, d, oct=oct)
        deltametric_vector = np.divide(deltametric_vector, len(atomIdx))
    else:
        deltametric_vector += deltametric(mol, w, atomIdx, d, oct=oct)
    return (deltametric_vector)

def atom_only_deltametric_derivative(mol, prop, d, atomIdx, oct=True,modifier=False):
    ## atomIdx must b either a list of indcies
    ## or a single index
    w = construct_property_vector(mol, prop, oct=oct,modifier=modifier)
    
    deltametric_derivative_mat = np.zeros((d + 1,mol.natoms))
    if hasattr(atomIdx, "__len__"):
        for elements in atomIdx:
            deltametric_derivative_mat += deltametric_derivative(mol, w, elements, d, oct=oct)
        deltametric_derivative_mat = np.divide(deltametric_derivative_mat, len(atomIdx))
    else:

        deltametric_derivative_mat += deltametric_derivative(mol, w, atomIdx, d, oct=oct)
    return (deltametric_derivative_mat)



def construct_property_vector(mol, prop, oct=True,modifier = False):
    ## assigns the value of property
    ## for atom i (zero index) in mol
    ## to position i in returned vector
    ## can be used to create weighted
    ## graph representations
    ## oct - bool, if complex is octahedral, will use better bond checks
    ## modifier - dict, used to modify prop vector (e.g. for adding 
    ##             ONLY used with  ox_nuclear_charge    ox or charge)
    ##              {"Fe":2, "Co": 3} etc
    allowed_strings = ['electronegativity', 'nuclear_charge', 'ident', 'topology',
                        'ox_nuclear_charge', 'size', 'vdwrad', 'effective_nuclear_charge']
    ## note that ident just codes every atom as one, this gives
    ## a purely toplogical index. coord gives the number of
    ## connecting atom to attom i (similar to Randic index)
    # if not oct:
    #     print('NOT using octahedral bonding pattern')
    globs = globalvars()
    prop_dict = dict()
    w = np.zeros(mol.natoms)
    done = False
    if not prop in allowed_strings:
        print('error, property  ' + str(prop) + ' is not a vaild choice')
        print(' options are  ' + str(allowed_strings))
        return False
    if prop == 'electronegativity':
        prop_dict = globs.endict()
    elif prop == 'size':
        at_keys = globs.amass().keys()
        for keys in at_keys:
            values = globs.amass()[keys][2]
            prop_dict.update({keys: values})
    elif prop == 'nuclear_charge':
        at_keys = globs.amass().keys()
        for keys in at_keys:
            values = globs.amass()[keys][1]
            prop_dict.update({keys: values})
    elif prop == 'effective_nuclear_charge': #Uses number of valence electrons
        if not modifier:
            at_keys = globs.amass().keys()
            for keys in at_keys:
                values = globs.amass()[keys][3]
                prop_dict.update({keys: values})
        else:
            at_keys = globs.amass().keys()
            for keys in at_keys:
                values = globs.amass()[keys][3]
                if keys in modifier.keys():
                    values += float(modifier[keys])
                prop_dict.update({keys: values})
    elif prop == 'ox_nuclear_charge':
        if not modifier:
            print('Error, must give modifier with ox_nuclear_charge')
            return False
        else:
            at_keys = globs.amass().keys()
            for keys in at_keys:
                values = globs.amass()[keys][1]
                if keys in modifier.keys():
                    values += float(modifier[keys])
                prop_dict.update({keys: values})
    elif prop == 'ident':
        at_keys = globs.amass().keys()
        for keys in at_keys:
            prop_dict.update({keys: 1})
    elif prop == 'topology':
        for i, atoms in enumerate(mol.getAtoms()):
            # print('atom # ' + str(i) + " symbol =  " + str(atoms.symbol()))
            w[i] = len(mol.getBondedAtomsSmart(i, oct=oct))
        done = True
    elif prop == 'vdwrad':
        prop_dict = globs.vdwrad()
        for i, atoms in enumerate(mol.getAtoms()):
            atom_type = atoms.symbol()
            if atom_type in globs.metalslist():
                w[i] = globs.amass()[atoms.symbol()][2]
            else:
                w[i] = prop_dict[atoms.symbol()]
        done = True
        # for keys in at_keys:
        #     prop_dict.update({keys: 1})
    if not done:
        for i, atoms in enumerate(mol.getAtoms()):
            # print('atom # ' + str(i) + " symbol =  " + str(atoms.symbol()))
            w[i] = prop_dict[atoms.symbol()]
    return (w)





def generate_full_complex_autocorrelations(mol, loud, depth=4, oct=True, flag_name=False, modifier=False):
    result = list()
    colnames = []
    allowed_strings = ['electronegativity', 'nuclear_charge', 'ident', 'topology', 'size', 'effective_nuclear_charge']
    labels_strings = ['chi', 'Z', 'I', 'T', 'S', 'Zeff']
    for ii, properties in enumerate(allowed_strings):
        metal_ac = full_autocorrelation(mol, properties, depth, oct=oct, modifier=modifier)
        this_colnames = []
        for i in range(0, depth + 1):
            this_colnames.append(labels_strings[ii] + '-' + str(i))
        colnames.append(this_colnames)
        result.append(metal_ac)
    if flag_name:
        results_dictionary = {'colnames': colnames, 'results_f_all': result}
    else:
        results_dictionary = {'colnames': colnames, 'results': result}
    return results_dictionary

def generate_full_complex_autocorrelation_derivatives(mol, loud, depth=4, oct=True, flag_name=False, modifier=False):
    result = None
    colnames = []
    allowed_strings = ['electronegativity', 'nuclear_charge', 'ident', 'topology', 'size', 'effective_nuclear_charge']
    labels_strings = ['chi', 'Z', 'I', 'T', 'S', 'Zeff']
    for ii, properties in enumerate(allowed_strings):
        f_ac_der = full_autocorrelation_derivative(mol, properties, depth, oct=oct, modifier=modifier)
        this_colnames = []
        for i in range(0, depth + 1):
            colnames.append(['d'+labels_strings[ii] + '-' + str(i)+ '/d' + labels_strings[ii] + str(j) for j in range(0, mol.natoms)])
        #colnames.append(this_colnames)
        if result is None:
            result = f_ac_der
        else:
            result = np.row_stack([result,f_ac_der])
    if flag_name:
        results_dictionary = {'colnames': colnames, 'results_f_all': result}
    else:
        results_dictionary = {'colnames': colnames, 'results': result}
    return results_dictionary


def generate_atomonly_autocorrelations(mol, atomIdx, loud, depth=4, oct=True):
    ## this function gets autocorrelations for a molecule starting
    ## in one single atom only
    # Inputs:
    #       mol - mol3D class
    #       atomIdx - int, index of atom3D class
    #       loud - bool, print output
    result = list()
    colnames = []
    allowed_strings = ['electronegativity', 'nuclear_charge', 'ident', 'topology', 'size', 'effective_nuclear_charge']
    labels_strings = ['chi', 'Z', 'I', 'T', 'S', 'Zeff']
    #print('The selected connection type is ' + str(mol.getAtom(atomIdx).symbol()))
    for ii, properties in enumerate(allowed_strings):
        atom_only_ac = atom_only_autocorrelation(mol, properties, depth, atomIdx, oct=oct)
        this_colnames = []
        for i in range(0, depth + 1):
            this_colnames.append(labels_strings[ii] + '-' + str(i))
        colnames.append(this_colnames)
        result.append(atom_only_ac)
    results_dictionary = {'colnames': colnames, 'results': result}
    return results_dictionary
    
def generate_atomonly_autocorrelation_derivatives(mol, atomIdx, loud, depth=4, oct=True):
    ## this function gets the d/dx for autocorrelations for a molecule starting
    ## in one single atom only
    # Inputs:
    #       mol - mol3D class
    #       atomIdx - int, index of atom3D class
    #       loud - bool, print output
    result = None
    colnames = []
    allowed_strings = ['electronegativity', 'nuclear_charge', 'ident', 'topology', 'size', 'effective_nuclear_charge']
    labels_strings = ['chi', 'Z', 'I', 'T', 'S', 'Zeff']
    #print('The selected connection type is ' + str(mol.getAtom(atomIdx).symbol()))
    for ii, properties in enumerate(allowed_strings):
        atom_only_ac = atom_only_autocorrelation_derivative(mol, properties, depth, atomIdx, oct=oct)
        this_colnames = []
        for i in range(0, depth + 1):
            colnames.append(['d'+labels_strings[ii] + '-' + str(i)+ '/d' + labels_strings[ii] + str(j) for j in range(0, mol.natoms)])
        if result is None:
            result = atom_only_ac
        else:
            result = np.row_stack([result,atom_only_ac])
    results_dictionary = {'colnames': colnames, 'results': result}
    return results_dictionary

def generate_atomonly_deltametrics(mol, atomIdx, loud, depth=4, oct=True):
    ## this function gets deltametrics for a molecule starting
    ## in one single atom only
    # Inputs:
    #       mol - mol3D class
    #       atomIdx - int, index of atom3D class
    #       loud - bool, print output
    result = list()
    colnames = []
    allowed_strings = ['electronegativity', 'nuclear_charge', 'ident', 'topology', 'size', 'effective_nuclear_charge']
    labels_strings = ['chi', 'Z', 'I', 'T', 'S', 'Zeff']
    # print('The selected connection type is ' + str(mol.getAtom(atomIdx).symbol()))
    for ii, properties in enumerate(allowed_strings):
        atom_only_ac = atom_only_deltametric(mol, properties, depth, atomIdx, oct=oct)
        this_colnames = []
        for i in range(0, depth + 1):
            this_colnames.append(labels_strings[ii] + '-' + str(i))
        colnames.append(this_colnames)
        result.append(atom_only_ac)
    results_dictionary = {'colnames': colnames, 'results': result}
    return results_dictionary
    
def generate_atomonly_deltametric_derivatives(mol, atomIdx, loud, depth=4, oct=True):
    ## this function gets deltametrics for a molecule starting
    ## in one single atom only
    # Inputs:
    #       mol - mol3D class
    #       atomIdx - int, index of atom3D class
    #       loud - bool, print output
    result = None
    colnames = []
    allowed_strings = ['electronegativity', 'nuclear_charge', 'ident', 'topology', 'size', 'effective_nuclear_charge']
    labels_strings = ['chi', 'Z', 'I', 'T', 'S', 'Zeff']
    # print('The selected connection type is ' + str(mol.getAtom(atomIdx).symbol()))
    for ii, properties in enumerate(allowed_strings):
        atom_only_ac_der = atom_only_deltametric_derivative(mol, properties, depth, atomIdx, oct=oct)
        this_colnames = []
        for i in range(0, depth + 1):
            colnames.append(['d'+labels_strings[ii] + '-' + str(i)+ '/d' + labels_strings[ii] + str(j) for j in range(0, mol.natoms)])
        if result is None:
            result = atom_only_ac_der
        else:
            result = np.row_stack([result,atom_only_ac_der])
    results_dictionary = {'colnames': colnames, 'results': result}
    return results_dictionary
